@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import os
 import random
@@ -19,9 +21,10 @@ from reviewtally.queries import (
     MAX_RETRIES,
     RETRYABLE_STATUS_CODES,
     SSL_CONTEXT,
+    build_github_rest_api_url,
+    require_github_token,
 )
 
-GITHUB_TOKEN = os.getenv("GITHUB_TOKEN")
 # get proxy settings from environment variables
 HTTPS_PROXY = os.getenv("HTTPS_PROXY")
 # check for lowercase https_proxy
@@ -81,9 +84,14 @@ async def check_rate_limit_and_sleep(response: aiohttp.ClientResponse) -> None:
         print(f"Warning: Could not parse rate limit headers: {e}")  # noqa: T201
 
 
-async def fetch(client: aiohttp.ClientSession, url: str) -> dict[str, Any]:
+async def fetch(
+    client: aiohttp.ClientSession,
+    url: str,
+    *,
+    github_token: str,
+) -> dict[str, Any]:
     headers = {
-        "Authorization": f"Bearer {GITHUB_TOKEN}",
+        "Authorization": f"Bearer {github_token}",
         "Accept": "application/vnd.github.v3+json",
     }
 
@@ -146,7 +154,12 @@ async def _backoff_delay(attempt: int) -> None:
     await asyncio.sleep(delay + jitter)
 
 
-async def fetch_batch(urls: list[str]) -> tuple[Any]:
+async def fetch_batch(
+    urls: list[str],
+    *,
+    github_token: str | None = None,
+) -> tuple[Any]:
+    token = github_token or require_github_token()
     connector = aiohttp.TCPConnector(
         ssl=SSL_CONTEXT,
         limit=CONNECTION_POOL_SIZE,
@@ -158,7 +171,10 @@ async def fetch_batch(urls: list[str]) -> tuple[Any]:
         timeout=AIOHTTP_TIMEOUT,
         connector=connector,
     ) as session:
-        tasks = [fetch(session, url) for url in urls]
+        tasks = [
+            fetch(session, url, github_token=token)
+            for url in urls
+        ]
         return await asyncio.gather(*tasks)  # type: ignore[return-value]
 
 
@@ -166,13 +182,17 @@ def get_reviewers_for_pull_requests(
     owner: str,
     repo: str,
     pull_numbers: list[int],
+    *,
+    github_token: str | None = None,
 ) -> list[dict]:
+    token = github_token or require_github_token()
     urls = [
-        f"https://api.github.com/repos/{owner}/{repo}"
-        f"/pulls/{pull_number}/reviews"
+        build_github_rest_api_url(
+            f"repos/{owner}/{repo}/pulls/{pull_number}/reviews",
+        )
         for pull_number in pull_numbers
     ]
-    reviewers = asyncio.run(fetch_batch(urls))
+    reviewers = asyncio.run(fetch_batch(urls, github_token=token))
     return [item["user"] for sublist in reviewers for item in sublist]
 
 
@@ -188,7 +208,9 @@ def _check_pr_cache(
 
     for pull_number in pull_numbers:
         cached_pr_data = cache_manager.get_cached_pr_review(
-            owner, repo, pull_number,
+            owner,
+            repo,
+            pull_number,
         )
         if cached_pr_data is not None:
             cached_results.extend(cached_pr_data)
@@ -202,14 +224,19 @@ def _fetch_review_metadata(
     owner: str,
     repo: str,
     uncached_prs: list[int],
+    *,
+    github_token: str,
 ) -> list[dict]:
     """Fetch reviews and collect metadata with comment URLs."""
     review_urls = [
-        f"https://api.github.com/repos/{owner}/{repo}"
-        f"/pulls/{pull_number}/reviews"
+        build_github_rest_api_url(
+            f"repos/{owner}/{repo}/pulls/{pull_number}/reviews",
+        )
         for pull_number in uncached_prs
     ]
-    reviews_response = asyncio.run(fetch_batch(review_urls))
+    reviews_response = asyncio.run(
+        fetch_batch(review_urls, github_token=github_token),
+    )
 
     review_data = []
 
@@ -219,10 +246,11 @@ def _fetch_review_metadata(
             user = review["user"]
             review_id = review["id"]
 
-            comment_url = (
-                f"https://api.github.com/repos/{owner}/{repo}"
-                f"/pulls/{pull_number}/reviews/{review_id}/comments"
+            comment_path = (
+                f"repos/{owner}/{repo}/pulls/{pull_number}/"
+                f"reviews/{review_id}/comments"
             )
+            comment_url = build_github_rest_api_url(comment_path)
 
             submitted_at = review.get("submitted_at")
             if submitted_at is None:
@@ -246,19 +274,22 @@ def _fetch_review_metadata(
 
 def _process_and_cache_reviews(
     cache_manager: CacheManager,
-    owner: str,
-    repo: str,
+    repo_identifier: tuple[str, str],
     review_data: list[dict],
     *,
+    github_token: str,
     use_cache: bool = True,
 ) -> list[dict]:
+    owner, repo = repo_identifier
     """Process comments and cache individual PR reviews."""
     if not review_data:
         return []
 
     # Extract comment URLs from review data
     comment_urls = [review["comment_url"] for review in review_data]
-    comments_response = asyncio.run(fetch_batch(comment_urls))
+    comments_response = asyncio.run(
+        fetch_batch(comment_urls, github_token=github_token),
+    )
 
     # Combine the data and group by PR for individual caching
     pr_review_data: dict[int, list[dict[str, Any]]] = {}
@@ -286,7 +317,10 @@ def _process_and_cache_reviews(
     if use_cache:
         for pull_number, reviews in pr_review_data.items():
             cache_manager.cache_per_review(
-                owner, repo, pull_number, reviews,
+                owner,
+                repo,
+                pull_number,
+                reviews,
             )
 
     return uncached_results
@@ -297,14 +331,19 @@ def get_reviewers_with_comments_for_pull_requests(
     repo: str,
     pull_numbers: list[int],
     *,
+    github_token: str | None = None,
     use_cache: bool = True,
 ) -> list[dict]:
     cache_manager = get_cache_manager()
+    token = github_token or require_github_token()
 
     if use_cache:
         # Check cache for each PR individually
         cached_results, uncached_prs = _check_pr_cache(
-            cache_manager, owner, repo, pull_numbers,
+            cache_manager,
+            owner,
+            repo,
+            pull_numbers,
         )
 
         # If all PRs are cached, return early
@@ -317,19 +356,29 @@ def get_reviewers_with_comments_for_pull_requests(
 
     # Fetch reviews and collect metadata
     review_data = _fetch_review_metadata(
-        owner, repo, uncached_prs,
+        owner,
+        repo,
+        uncached_prs,
+        github_token=token,
     )
 
     # Process comments and cache results
     uncached_results = _process_and_cache_reviews(
-        cache_manager, owner, repo, review_data, use_cache=use_cache,
+        cache_manager,
+        (owner, repo),
+        review_data,
+        github_token=token,
+        use_cache=use_cache,
     )
 
     # Cache empty results for PRs with no reviews (only if caching enabled)
     if not review_data and use_cache:
         for pull_number in uncached_prs:
             cache_manager.cache_per_review(
-                owner, repo, pull_number, [],
+                owner,
+                repo,
+                pull_number,
+                [],
             )
 
     # Combine cached and newly fetched results
